@@ -18,6 +18,8 @@
 #include "epos.h"
 #include "hashtmpl.h"
 
+// #include <netdb.h>
+
 /*
  *	This is a good place for hash table instantiation needed by both
  *	the monolith and server code.
@@ -31,10 +33,12 @@
 #ifdef HAVE_TEMPL_INST
 	template class hash_table<char, file>;
 	template class hash_table<char, epos_option>;
+//	template class hash_table<char, hostent>;
 	template class hash_table<wchar, wchar>;
-#else	// else it is WatcomC
+#else	// e.g., a prehistoric WatcomC
 	extern hash_table<char, file> *_dummy_freadin_hash_tmpl_inst;	// remove this one asterisk if Watcom barfs, else remove this comment
 	extern hash_table<char, epos_option> *_dummy_option_hash_tmp_inst;
+//	extern hash_table<char, hostent> *_dummy_hostent_hash_tmp_inst;
 	extern hash_table<wchar, wchar> *_dummy_char_hash_tmpl_inst;
 #endif
 
