@@ -87,7 +87,7 @@ void cow(cowabilium **p, int size, int extraoffset, int extrasize)
 	}
 }
 
-#define  LANGS_OFFSET   ((int)&((configuration *)NULL)->langs)
+#define  LANGS_OFFSET   ((long int)&((configuration *)NULL)->langs)
 #define  LANGS_LENGTH   ((*cfg)->n_langs * sizeof(void *))
 
 void cow_configuration(configuration **cfg)
@@ -295,7 +295,7 @@ void restrict_options()
 		delete t;
 		return;		/* if no restrictions file, ignore it */
 	}
-	char *line = (char *)xmalloc(scfg->max_line_len);
+	char *line = get_text_line_buffer();
 	while (t->get_line(line)) {
 		char *status = split_string(line);
 		o = option_set->translate(line);
@@ -467,13 +467,35 @@ static inline void post_set_unit_levels(const char *)
 	}
 }
 
+static inline char *get_startup_cwd(char *buff, size_t size)
+{
+	static char *startup_wd = NULL;
+	if (startup_wd != NULL) {
+		if (strlen(startup_wd) + 1 <= size) {
+			strcpy(buff, startup_wd);
+			return startup_wd;
+		} else {
+			return NULL;
+		}
+	}
+	char *ret = getcwd(buff, size);
+	if (ret == NULL) {
+		return NULL;
+	}
+	startup_wd = FOREVER(strdup(ret));
+	return ret;
+}
+
 static inline const char *pre_base_dir(const char *value)
 {
-	if (value && value[0] != SLASH) {
+	if (value && value[0] && value[0] != SLASH) {
 		size_t size = scfg->scratch_size ? scfg->scratch_size : 64;
 		char *abs = (char *)xmalloc(size);
-		while (!getcwd(abs, size - 1))
+		while (!get_startup_cwd(abs, size - 1))
 			size *= 2, abs = (char *)xrealloc(abs, size);
+		if (!strcmp(abs, "/")) {
+			shriek(814, "base_dir relative to the root? Prepend the slash.");
+		}
 		while (strlen(abs) + strlen(value) + 1 >= size)
 			size *= 2, abs = (char *)xrealloc(abs, size);
 		char *tmp = abs + strlen(abs);
@@ -651,7 +673,7 @@ bool set_option(epos_option *o, const char *val, void *base)
 			D_PRINT(1, "Configuration option \"%s\" adds \"%s\"%s to \"%s\"%s\n", o->optname,
 					val, strchr(val, '\033') && scfg->normal_color ? scfg->normal_color : "",
 					old ? old : "empty string", strchr(val, '\033') && scfg->normal_color ? scfg->normal_color : "");
-			if (!old) {
+			if (!old || !*old) {
 				*(char **)locus = strdup(val);
 				break;
 			}
@@ -678,10 +700,15 @@ bool set_option(epos_option *o, const char *val, void *base)
 			break;
 		case O_CHARSET:
 			parse_cfg_str(const_cast<char *>(val));
-			if ((tmp = load_charset(val)) == CHARSET_NOT_AVAILABLE)
+			if (!strncmp(val, "sampa-", 6)) {
+				tmp = load_named_sampa(val);
+			} else {
+				tmp = load_charset(val);
+			}
+			if (tmp == CHARSET_NOT_AVAILABLE)
 				shriek(447, "Unknown charset %s", val);
 			*(int *)locus = tmp;
-			D_PRINT(1, "Charset set to %i\n",*(int*)locus);
+			D_PRINT(2, "Charset set to %i\n",*(int*)locus);
 			break;
 		default: shriek(462, "Bad option type %d", (int)o->opttype);
 	}
@@ -727,7 +754,7 @@ bool set_option_to_default(epos_option *o, void *base)
  *	The order of cowing cfg, lang and voice, is important.
  */
 
-#define  VOICES_OFFSET  ((int)&((lang *)NULL)->voicetab)
+#define  VOICES_OFFSET  ((long int)&((lang *)NULL)->voicetab)
 #define  VOICES_LENGTH  (this_lang->n_voices * sizeof(void *))
 
 bool set_option(epos_option *o, const char *value)
@@ -748,7 +775,7 @@ bool set_option(epos_option *o, const char *value)
 	return false;
 }
 
-static inline bool set_option(char *name, const char *value)
+bool set_option(char *name, const char *value)
 {
 	return set_option(option_struct(name, NULL), value);
 }
@@ -798,7 +825,7 @@ bool voice_switch(const char *value)
 	return false;
 }
 
-const char *format_option(epos_option *o, void *base)
+static const char *format_option(epos_option *o, void *base)
 {
 	char *locus = (char *)base + o->offset;
 	switch(o->opttype) {
@@ -829,7 +856,7 @@ const char *format_option(epos_option *o, void *base)
 		case O_VOICE:
 			return (char *)this_voice->name;
 		case O_CHARSET:
-			return enum2str(*(int *)locus, charset_list);
+			return get_charset_name(*(int *)locus);
 		default: shriek(462, "Bad option type %d", (int)o->opttype);
 	}
 	return NULL; /* unreachable */
@@ -881,8 +908,8 @@ void reinitialize_configuration()
 	set_option_or_die("fixed_ini_file", "fixed.ini");
 	set_option_or_die("restr_file", "restr.ini");
 
-	set_option_or_die("_token_esc", "nrt[eE\\ #;@~.d-mXYZWVU");
-	set_option_or_die("_value_esc", "\n\r\t\033\033\033\\\377#;@\1\2\3\4\5\037\036\035\034\032\030");
+	set_option("_token_esc", "nrt[eE\\ #;@~.d-mXYZWVU");		/* no ..._or_die just because of the leading underscore */
+	set_option("_value_esc", "\n\r\t\033\033\033\\\377#;@\1\2\3\4\5\037\036\035\034\032\030");
 
 	scfg->_slash_esc = '/';
 
@@ -901,6 +928,26 @@ void reinitialize_configuration()
 int argc_copy = 0;		// these get filled by the args to main()
 char **argv_copy = NULL;
 
+void set_cmd_line(int argc, char **argv)
+{
+	argc_copy = argc;
+	char **args = (char **)xmalloc(argc * (sizeof(char *) + 1));
+	for (int i = 0; i < argc; i++) {
+		args[i] = get_text_buffer(strlen(argv[i]));
+		strcpy(args[i], argv[i]);
+	}
+	args[argc] = NULL;
+	argv_copy = args;
+}
+
+void free_cmd_line()
+{
+	for (int i = 0; i < argc_copy; i++) {
+		free(argv_copy[i]);
+	}
+	free(argv_copy);
+}
+
 void parse_cmd_line()
 {
 	char *ar;
@@ -908,38 +955,38 @@ void parse_cmd_line()
 	register char *par;
 	int dees = 0;
 
-	for(int i=1; i<argc_copy; i++) {
+	for(int i = 1; i < argc_copy; i++) {
 		j = ar = argv_copy[i];
 		encode_string(ar, cfg->charset, true);
 		switch(strspn(ar, CMD_LINE_OPT)) {
 		case 3:
-			ar+=3;
+			ar += 3;
 			if (strchr(ar, CMD_LINE_VAL) && scfg->_warnings) 
 				shriek(814, "Thrice dashed options have an implicit value");
 			set_option_or_die(ar, "0");
 			break;
 		case 2:
-			ar+=2;
-			par=strchr(ar, CMD_LINE_VAL);
+			ar += 2;
+			par = strchr(ar, CMD_LINE_VAL);
 			if (par) {					//opt=val
-				*par=0;
-				set_option_or_die(ar, par+1);
-				*par=CMD_LINE_VAL;
-			} else	if (i+1==argc_copy || strchr(CMD_LINE_OPT, *argv_copy[i+1])) 
+				*par = 0;
+				set_option_or_die(ar, par + 1);
+				*par = CMD_LINE_VAL;
+			} else	if (i + 1 == argc_copy || strchr(CMD_LINE_OPT, *argv_copy[i+1])) 
 					set_option_or_die(ar, "");
 				else set_option_or_die(ar, argv_copy[++i]);
 			break;
 		case 1:
-			for (j = ar+1; *j; j++) switch (*j) {
-				case 'b': scfg->structured=false; break;
-//				case 'd': scfg->show_seg=true; break;
-//				case 'e': scfg->show_phones=true; break;
-				case 'f': scfg->forking=false; break;
-				case 'H': scfg->long_help=true;	/* fall through */
-				case 'h': scfg->help=true; break;
-				case 'p': scfg->pausing=true; break;
-				case 's': scfg->play_segments=true; break;
-				case 'v': scfg->version=true; break;
+			for (j = ar + 1; *j; j++) switch (*j) {
+				case 'b': scfg->structured = false; break;
+//				case 'd': scfg->show_seg = true; break;
+//				case 'e': scfg->show_phones = true; break;
+				case 'f': scfg->forking = false; break;
+				case 'H': scfg->long_help = true;	/* fall through */
+				case 'h': scfg->help = true; break;
+				case 'p': scfg->pausing = true; break;
+				case 's': scfg->play_segments = true; break;
+				case 'v': scfg->version = true; break;
 				case 'D':
 					if (!scfg->debug) scfg->debug = true;
 					dees++;
@@ -980,7 +1027,7 @@ void parse_cmd_line()
 			}
 			break;
 		case 0:
-			if (!is_monolith) shriek(814, "Only options allowed at Epos server command line\nUse a client (e.g. \"say\") to specify text");
+			if (!is_monolith) shriek(814, "Only options allowed at Epos server command line\nUse a client (e.g. \"say-epos\") to specify text");
 			if (scfg->_input_text && scfg->_input_text != ar) {
 				if (!scfg->_warnings) break;
 				if (cfg->paranoid) shriek(814, "Quotes forgotten on the command line?");
@@ -1004,7 +1051,7 @@ void load_config(const char *filename, const char *dirname, const char *what,
 
 	if (!filename || !*filename) return;
 	D_PRINT(3, "Loading %s from %s\n", what, filename);
-	char *line = (char *)xmalloc(scfg->max_line_len + 2) + 2;
+	char *line = get_text_line_buffer() + 2;
 	line[-2] = OPT_STRUCT_PREFIX[type];
 	line[-1] = ':';
 	text *t = new text(filename, dirname, "", what, true);
@@ -1113,7 +1160,7 @@ static inline void dump_help()
 
 	if (is_monolith) {
 		printf(" You probably don't want to use this monolithic binary.\n");
-		printf(" The usage is somewhere between 'say' and 'epos',\n");
+		printf(" Its usage resembles 'say-epos' and 'eposd',\n");
 		printf(" but the details are not documented, maintained nor supported.\n");
 		exit(0);
 	}
@@ -1123,8 +1170,8 @@ static inline void dump_help()
 	printf(" -p  pausing - show every intermediate state\n");
 	printf(" -v  show version\n");
 	printf(" -D  debugging info (more D's - more detailed)\n");
-	printf(" --some_long_option    ...see src/options.lst or 'epos -H' for these\n");
-	printf("(use one of the clients (e.g. 'say') to pass input to Epos)\n");
+	printf(" --some_long_option    ...see src/options.lst or 'eposd -H' for these\n");
+	printf("(use one of the clients (e.g. 'say-epos') to pass input to Epos)\n");
 	if (!scfg->long_help) exit(0);
 
 	printf("Long option types:\n");
@@ -1155,9 +1202,7 @@ void config_init()
 {
 	const char *mlinis[] = {"","ansi.ini","rtf.ini", NULL};
 
-//	if (option_set) shriek(862, "config_init() too late");
 	make_option_set();
-//	restrict_options();
 //	if (privileged_exec())	/* parse restr.ini before command line */
 //		restrict_options(); 
 	parse_cmd_line();
@@ -1181,8 +1226,6 @@ void config_init()
 	parse_cmd_line();
 	scfg->_loaded=true;
 	
-// 	hash_max_line = scfg->max_line_len;
-
 	if (scfg->version) version();
 	if (scfg->help || scfg->long_help) dump_help();
 }
