@@ -110,6 +110,7 @@ epos_option *option_struct(const char *name, hash_table<char, epos_option> *soft
    backslashes or double quotes  		*/
 bool set_option(epos_option *o, const char *value);			// the const qualifier IS A LIE
 bool set_option(epos_option *o, const char *value, void *whither);	// the const qualifier IS A LIE
+bool set_option(char *name, const char *value);				// the const qualifier IS A LIE
 
 
 const char *format_option(epos_option *name);	// may return scratch etc.
@@ -136,7 +137,7 @@ void free_extra_options();
 
 void reinitialize_configuration();
 
-extern int argc_copy;
-extern char **argv_copy;
+void set_cmd_line(int argc, char **argv);
+void free_cmd_line();
 
 #define DQUOT          '"'            //used when parsing the .ini file
