@@ -42,7 +42,6 @@ unit::unit(UNIT layer, parser *parser)
 	next = prev = firstborn = lastborn = father = NULL;
 	depth = layer;
 	cont = NO_CONT;
-//	f = i = t = 0;
 	t = 1;
 	m = NULL;
 	scope = false;
@@ -50,10 +49,6 @@ unit::unit(UNIT layer, parser *parser)
 
 	while (parser->level < layer) insert_end(new unit((UNIT) (layer-1), parser),NULL);
 	if    (parser->level == layer) {
-//		shriek(899, "unit const");
-//		m =
-//		f = parser->f;
-//		i = parser->i;
 		t = parser->t;
 		cont = parser->gettoken();
 	}
@@ -786,13 +781,16 @@ unit::relabel(hash *table, regex_t *fastmatch, SUBST_METHOD method, UNIT target)
 				for (char *p = gb + len - i; p >= gb; p--) {
 					char tmp = p[i];
 					p[i] = 0;
+					D_PRINT(0, "Try recoding a substring: %s\n", p);
 					r = table->translate(p);
 					p[i] = tmp;
 					if (r) {
-						i -= (p == gb);
-						p += (p == gb);
-						if (cfg->paranoid && strlen(r) - i)
-							shriek(462, "Substitute length differs: beginning of '%s' to '%s'", p, r);
+						i -= (p == gb);			// ^
+						p += (p == gb);			// ^
+						i -= (p + i == gb + len);	// $
+						D_PRINT(0, "Match found: %s, length %d\n", r, i);
+						if (strlen(r) - i)
+							shriek(462, "Substitute length differs: '%s' to '%s'", p, r);
 						memcpy(p, r, strlen(r));
 						goto break_home;
 					}
@@ -938,8 +936,10 @@ unit::assim(UNIT target, bool backwards, charxlat *fn, charclass *left, charclas
 void 
 unit::split(unit *before)
 {
-	if (before->father != this) {
-		before->father->split(before);
+	if (before->father != this) {	// this handles multi-level splits
+		if (before->prev) {
+			before->father->split(before);
+		}
 		split(before->father);
 		return;
 	}
@@ -950,7 +950,7 @@ unit::split(unit *before)
 	DBG(1, fout(NULL);)
 	clone->scope=false;
 	if(!(lastborn=before->prev))
-		shriek (463, "Attempted splitting a unit just at its boundary");
+		shriek (463, "Attempted to split a unit at its present boundary");
 	before->prev=NULL;
 	lastborn->next=NULL;
 	clone->firstborn=before;
@@ -981,19 +981,30 @@ char syll_pending;
 void
 unit::syll_break(char *sonor, unit *before)
 {
-	father->split(before);
-	syll_pending=0;
+	unit *ancestor = this;
+	while (ancestor->father && !ancestor->scope) {
+		ancestor = ancestor->father;
+	}
+	ancestor->split(before);
+	D_PRINT(0, "Made the split\n");
+	syll_pending = 0;
 }
 
 void 
 unit::syllabify(char *sonor)
 {
-	if (sonor[inside()]<sonor[Next(depth)->inside()])
-		if (sonor[inside()]<sonor[Prev(depth)->inside()]) syll_break(sonor,this);
-		else syll_pending+=(sonor[inside()]==sonor[Prev(depth)->inside()]);
-	else if (syll_pending && sonor[inside()]<sonor[Prev(depth)->inside()]) {
-		syll_break(sonor,next);
-		syll_pending=0;
+	if (sonor[inside()]<sonor[Next(depth)->inside()]) {
+		if (sonor[inside()]<sonor[Prev(depth)->inside()]) {
+			D_PRINT(0, "Immediate decision to split before a sharp dropdown\n");
+			syll_break(sonor,this);
+		} else {
+			bool equisonorous = sonor[inside()] == sonor[Prev(depth)->inside()];
+			syll_pending += equisonorous;
+			D_PRINT(0, "Postponing the decision to split\n", syll_pending);
+		}
+	} else if (syll_pending && sonor[inside()]<sonor[Prev(depth)->inside()]) {
+		D_PRINT(0, "Decision to split after the 1st char of a %d chars long dropdown\n", syll_pending + 1);
+		syll_break(sonor, Next(depth));
 	}
 }
 
@@ -1002,13 +1013,16 @@ unit::syllabify(UNIT target, char *sonor)
 {
 	unit *tmpu, *tmpu_prev;
 	
-	D_PRINT(0, "unit::syllabify in level %d cont %c %c %c \n", depth,Prev(depth)->cont,cont,Next(depth)->cont);
+	D_PRINT(1, "unit::syllabify in level %d cont %c %c %c \n", depth,Prev(depth)->cont,cont,Next(depth)->cont);
 	syll_pending=0;
 	for (tmpu=RightMost(target);tmpu!=&EMPTY; ) {
+		D_PRINT(0, "Considering a target unit, cont %c %c %c\n", tmpu->Prev(depth)->cont, tmpu->cont, tmpu->Next(depth)->cont);
 		tmpu_prev = tmpu->Prev(tmpu->depth);
 		tmpu->syllabify(sonor);
 		tmpu = tmpu_prev;
+		D_PRINT(0, "Considering the next target unit\n");
 	}
+	D_PRINT(0, "Syllabification applied for a whole scope unit.  syll_pending=%d\n", syll_pending);
 }
 
 const int INFINITE_BADNESS = 300000000;
@@ -1075,7 +1089,7 @@ unit::analyze(UNIT target, hash *table, int unanal_unit_penalty, int unanal_part
 	for (unit *u = LeftMost(target); u != &EMPTY; u = u->Next(target)) {
 		if (count == next_split) {
 			D_PRINT(0, "Morphoanalyzer splits at %d\n", next_split);
-			if (top->next) shriek(899, "Hu.");
+			if (top->next) shriek(461, "Already split");
 			top->split(u);
 			top = top->next;
 			next_split = vb[count].badness;
@@ -1098,16 +1112,32 @@ unit::contains(UNIT target, charclass *set)
 }
 
 void
-unit::absol(UNIT target)
+unit::absol(hash *dict, UNIT target)
 {
+#ifdef LAME_ABSOLUTIZATION
 	if (target == depth) {
 		if (strchr("aeiou", cont)) t *= 1.3;
 		if (strchr("AEO", cont)) t *= 1.7;
 		if (strchr("ptkbdg", cont)) t *= 0.9;
 	} else if (target < depth) {
 		for (unit *u = firstborn; u; u = u->next)
-			u->absol(target);
-	} else shriek(899, "absolutize upwards");
+			u->absol(dict, target);
+	} else shriek(463, "Tried to absolutize upwards");
+#else
+	if (target == depth) {
+		char req[2];
+		req[1] = 0;
+		req[0] = cont;
+		int timing = dict->translate_int(req);
+		if (timing == -1) {
+			shriek(469, "No timings?  FIXME");
+		}
+		t = t * timing / 100;	// FIXME 100
+	} else if (target < depth) {
+		for (unit *u = firstborn; u; u = u->next)
+			u->absol(dict, target);
+	} else shriek(463, "Tried to absolutize upwards");
+#endif
 	
 }
 
@@ -1258,7 +1288,6 @@ unit::smooth(UNIT target, int *recipe, int n, int rec_len, FIT_IDX what)
 		if (u->Next(target) != &EMPTY) u = u->Next(target);
 	}
 #else
-//	shriek(899, "no smooth");
 	D_PRINT(3, "No smooth!\n");	// FIXME
 #endif
 }
@@ -1277,7 +1306,7 @@ unit::project_extents()
 			shriek(862, "Extent followed by a non-extent");
 
 	for (unit *u = firstborn; u; u = u->next) {
-		D_PRINT(2, "Moving prosody point to %d; q=%d, val=%d\n", u->depth, m->quant, m->par);
+		D_PRINT(1, "Moving prosody point to %d; q=%d, val=%d\n", u->depth, m->quant, m->par);
 		sanity();
 		u->sanity();
 		m->derived()->merge(u->m);
@@ -1333,7 +1362,7 @@ unit::project(UNIT target)			// FIXME optimize
 		project_one_level(sum);
 		for (u = firstborn; u; u = u->next)
 			u->project(target);
-	} else shriek(899, "project upwards");
+	} else shriek(463, "Tried to project upwards");
 }
 
 /****************************************************************************
@@ -1596,10 +1625,10 @@ unit::do_sanity()
 	if (lastborn && lastborn->prev && !m->disjoint(lastborn->prev->m)) insane("disjointness problem");
 
         if (scfg->ptr_trusted) return;
-	if (prev && (unsigned int) prev<0x8000000) insane("prev");
-	if (next && (unsigned int) next<0x8000000)  insane("next");
-	if (firstborn && (unsigned int) firstborn<0x8000000) insane("firstborn");
-	if (lastborn && (unsigned int) lastborn<0x8000000)  insane("lastborn");
+	if (prev && (unsigned long) prev<0x8000000) insane("prev");
+	if (next && (unsigned long) next<0x8000000)  insane("next");
+	if (firstborn && (unsigned long) firstborn<0x8000000) insane("firstborn");
+	if (lastborn && (unsigned long) lastborn<0x8000000)  insane("lastborn");
 }
 
 void
